#!/bin/env python
# -*- coding: utf-8 -*-

import difflib
import sys
import argparse
import webbrowser
import os
import filecmp
import subprocess

from colorama import Fore, init, AnsiToWin32
init(wrap=False)
stream = AnsiToWin32(sys.stderr).stream

compResultFile = "comp_result.html"
notebookFile = "notebook.ipynb"

# 读取文件
def read_file(file_name):
    try:
        file_desc = open(file_name, 'r', encoding='utf-8')
        # 读取后按行分割
        text = file_desc.read().splitlines()
        file_desc.close()
        return text
    except IOError as error:
        print('Read input file Error: {0}'.format(error))
        sys.exit()

# 比较时忽略行末的空格和文件末尾的回车换行
def advanced_file_compare(file1, file2):
    f1 = open(file1, encoding='utf-8')
    f2 = open(file2, encoding='utf-8')
   
    returnVal = 1

    str1 = []
    for line1 in f1:
        line1 = line1.rstrip()
        line1 = line1.replace('\n', '')
        if len(line1) != 0:
            str1.append(line1)
    str2 = []
    for line2 in f2:
        line2 = line2.rstrip()
        line2 = line2.replace('\n', '')
        if len(line2) != 0:
            str2.append(line2)
    
    count = 0
    if len(str1) == len(str2):
        for line1 in str1:
            if line1 != str2[count]: #文件不同
                returnVal = 0
                break
            else:
                count = count + 1
    else:
        returnVal = 0
           
    f1.close()
    f2.close()
   
    return returnVal

# 比较两个文件并把结果生成一份html文本
def compare_file(file1, file2, seqNum, type):
    returnVal = 0
    if file1 == "" or file2 == "":
        print('文件路径不能为空：第一个文件的路径：{0}, 第二个文件的路径：{1} .'.format(file1, file2))
        sys.exit()
    else:
        print("正在将标准查询结果文件 {0} 与用户编写的查询语句查询结果 {1} 进行比较。\n比较结果".format(file1, file2), end=': ')
    
#    if os.path.isfile(file1) and os.path.isfile(file2) and filecmp.cmp(file1, file2):
    if os.path.isfile(file1) and os.path.isfile(file2) and advanced_file_compare(file1, file2):
        print("文件相同")
        if type == 1:
            print(Fore.GREEN + "query{0}.sql 验证成功".format(seqNum), file = stream, end='')
        else:
            print(Fore.GREEN + "schema_check{0}.sql 验证成功".format(seqNum), file = stream, end='')
        print(Fore.WHITE, file = stream)
        returnVal = 1
        return returnVal
    else:
        print("文件不同")
        if type == 1:
            print(Fore.RED + "query{0}.sql 验证失败".format(seqNum), file = stream, end='')
        else:
            print(Fore.RED + "schema_check{0}.sql 验证失败".format(seqNum), file = stream, end='')
        print(Fore.WHITE, file = stream)

    text1_lines = read_file(file1)
    text2_lines = read_file(file2)
    diff = difflib.HtmlDiff()    # 创建HtmlDiff 对象
    result = diff.make_file(text1_lines, text2_lines)  # 通过make_file 方法输出 html 格式的对比结果
    # 将结果写入到比较结果文件中
    try:
        with open(compResultFile, 'a+', encoding="utf-8") as result_file:
            if type ==1:
                promptContent = "<p>query{0}.sql 验证失败。</br>正在将标准查询结果文件 query{0}_result.txt（左边）与用户编写的查询语句输出结果 user_query{0}_result.txt（右边）进行比较：</p>".format(seqNum)
            else:
                promptContent = "<p>schema_check{0}.sql 验证失败。</br>正在将标准查询结果文件 schema_check{0}_result.txt（左边）与查询语句输出结果 user_schema_check{0}_result.txt（右边）进行比较：</p>".format(seqNum)
            result = promptContent + result
            result_file.write(result)
    except IOError as error:
        print('写入 html 文件错误：{0}'.format(error))
    finally:
        return returnVal


if __name__ == "__main__":
    print()
    print(Fore.YELLOW + "提示：\n1. 启用验证前一定要先保存 sql 文件。\n2. 每次启动验证，均会重新创建数据库、数据表和插入数据。", file = stream)
    print(Fore.WHITE, file = stream)
    
    # 移除比较结果文件
    if os.path.isfile(compResultFile):
        os.remove(compResultFile)

    # 执行每一个query查询
    seqNum = 1
    while 1:
        outputFile = "query{0}_result.txt".format(seqNum)
        useroutputFile = "user_query{0}_result.txt".format(seqNum)
        if seqNum == 1 and not os.path.isfile(outputFile):
            print(Fore.RED + "该项目未提供自动化验证功能", file = stream)
            exit(1)
        if not os.path.isfile(outputFile):
            break
        print("\n正在使用 query{0}.sql 进行查询，结果保存到文件 {1}。".format(seqNum, useroutputFile))
        # 将对应的case号当参数传入
        runCommand = "python testsql.py local_test query {0}".format(seqNum)
        execResult = os.system(runCommand)
        if execResult != 0:
            print(Fore.RED + "应用程序异常，返回值：{0}".format(execResult), file = stream)
            exit(1)
 
        if os.path.isfile(outputFile) and os.path.isfile(useroutputFile):
            if compare_file(outputFile, useroutputFile, seqNum, 1) == 0:
                print("正在使用浏览器打开文件比较结果，可帮助用户查找验证失败的原因。")
                # 使用浏览器打开比较结果页面
                webbrowser.open('file://' + os.path.realpath(compResultFile))
                # 只要有验证失败的，就不再继续检查
                exit(1)
        seqNum = seqNum + 1

    # 执行每一个schema_check查询表结构
    seqNum = 1
    while 1:
        outputFile = "schema_check{0}_result.txt".format(seqNum)
        useroutputFile = "user_schema_check{0}_result.txt".format(seqNum)
        if seqNum == 1 and not os.path.isfile(outputFile):
            print(Fore.RED + "该项目未提供自动化验证功能", file = stream)
            exit(1)
        if not os.path.isfile(outputFile):
            break
        print("\n正在使用schema_check{0}.sql进行表结构查询，查询结果保存到文件 {1}。".format(seqNum, useroutputFile))
        # 将对应的case号当参数传入
        runCommand = "python testsql.py local_test schema_check {0}".format(seqNum)
        execResult = os.system(runCommand)
        if execResult != 0:
            print(Fore.RED + "应用程序异常，返回值：{0}".format(execResult), file = stream)
            exit(1)
 
        if os.path.isfile(outputFile) and os.path.isfile(useroutputFile):
            if compare_file(outputFile, useroutputFile, seqNum, 2) == 0:
                print("正在使用浏览器打开文件比较结果，可帮助用户查找验证失败的原因。")
                # 使用浏览器打开比较结果页面
                webbrowser.open('file://' + os.path.realpath(compResultFile))
                break
        seqNum = seqNum + 1    
